package gov.llnl.babel.ast;

import gov.llnl.babel.parsers.sidl2.ParseTreeNode;

/**
 * Base class for user defined types such as <code>Class</code>,
 * <code>Interface</code>, <code>Enumeration</code> and
 * <code>Package</code>.
 */
public abstract class NamedType extends Type implements INameable {

  protected Name d_name = null;

  protected DocComment d_comment = null;

  public NamedType(ParseTreeNode src, ASTNode parent, Name name) {
    super(src, parent);
    d_name = name;
    setDocComment(src.doc);
  }

  public Name getName() {
    return d_name;
  }

  public void setName(Name name) {
    d_name = name;
  }

  public boolean hasDocComment() {
    return d_comment != null;
  }

  public DocComment getDocComment() {
    return d_comment;
  }

  public void setDocComment(String text) {
    if (text == null || text.length() == 0) {
      d_comment = null;
    } else {
      d_comment = new DocComment(text);
    }
  }

  /** get Fully Qualified Name by traversing parent heirarchy */
  public String getFQN() {
    if (d_parent != null) {
      if (d_parent instanceof NamedType) {
        return ((NamedType) d_parent).getFQN() + "." + d_name.toString();
      } else if (d_parent instanceof SIDLFile) {
        return "." + d_name.toString();
      }
    }
    return d_name.toString();
  }

  public Version getVersion() {
    if (d_parent != null) {
      if (d_parent instanceof NamedType) {
        return ((NamedType) d_parent).getVersion();
      }
    }
    return null;
  }
}
