package gov.llnl.babel.visitor;

import gov.llnl.babel.ast.Argument;
import gov.llnl.babel.ast.ArgumentList;
import gov.llnl.babel.ast.ArrayType;
import gov.llnl.babel.ast.RArrayType;
import gov.llnl.babel.ast.Extents;
import gov.llnl.babel.ast.Attribute;
import gov.llnl.babel.ast.AttributeList;
import gov.llnl.babel.ast.ClassType;
import gov.llnl.babel.ast.DocComment;
import gov.llnl.babel.ast.Enumeration;
import gov.llnl.babel.ast.EnumItem;
import gov.llnl.babel.ast.FixedType;
import gov.llnl.babel.ast.ImportClause;
import gov.llnl.babel.ast.InterfaceType;
import gov.llnl.babel.ast.Method;
import gov.llnl.babel.ast.MethodList;
import gov.llnl.babel.ast.NamedType;
import gov.llnl.babel.ast.ASTNode;
import gov.llnl.babel.ast.Package;
import gov.llnl.babel.ast.RequireClause;
import gov.llnl.babel.ast.SIDLFile;
import gov.llnl.babel.ast.TypeSearchClause;
import gov.llnl.babel.ast.ScopedID;
import gov.llnl.babel.ast.ScopedIDList;
import gov.llnl.babel.ast.Type;
import gov.llnl.babel.ast.ThrowsList;
import gov.llnl.babel.ast.FromClause;
import gov.llnl.babel.ast.StructType;
import gov.llnl.babel.ast.StructItem;
import gov.llnl.babel.ast.Name;

import java.util.Iterator;

/**
 * 
 * Base class for all visitors over gov.llnl.babel.ast.*
 * 
 * This default implementation simply iterates over everything. Use as a base
 * class and simply override the parts that you are interested in
 */
public class Visitor {

	public Object visitNode(ASTNode node, Object data) {
		return data;
	}

	public Object visitSIDLFile(SIDLFile node, Object data) {
		for (Iterator i = node.getTypeSearchList().iterator(); i.hasNext();) {
			TypeSearchClause tsc = (TypeSearchClause) i.next();
			tsc.accept(this, data);
		}
		for (Iterator i = node.getPackageList().iterator(); i.hasNext();) {
			Package p = (Package) i.next();
			p.accept(this, data);
		}
		return data;
	}

	public Object visitImportClause(ImportClause node, Object data) {
		return data;
	}

	public Object visitRequireClause(RequireClause node, Object data) {
		return data;
	}

	public Object visitPackage(Package node, Object data) {
		for (Iterator i = node.getNamedTypes().iterator(); i.hasNext();) {
			NamedType t = (NamedType) i.next();
			t.accept(this, data);
		}
		return data;
	}

  public Object visitAttribute( Attribute node, Object data ) { 
    return data;
  }
  
  public Object visitAttributeList( AttributeList node, Object data ) { 
    for (Iterator i = node.iterator(); i.hasNext(); ) {
      Attribute a = (Attribute) i.next();
      a.accept(this, data);
    }
    return data;
  }
  
  public Object visitEnumeration( Enumeration node, Object data ) {
    for(Iterator i = node.getEnumItemList().iterator(); i.hasNext(); ) { 
      EnumItem e = (EnumItem) i.next();
      e.accept(this, data);
    }
    return data;
  }
  

  public Object visitEnumItem( EnumItem node, Object data ) {
    return data;
  }
  
	public Object visitClassType(ClassType node, Object data) {
    MethodList ml = node.getMethodList();
    ml.accept(this, data);
		return data;
	}

	public Object visitInterfaceType(InterfaceType node, Object data) {
    MethodList ml = node.getMethodList();
    ml.accept(this,data);
		return data;
	}

	public Object visitArrayType(ArrayType node, Object data) {
		return data;
	}

	public Object visitRArrayType(RArrayType node, Object data) {
		return data;
	}

  public Object visitExtents(Extents node, Object data) { 
    for(Iterator it=node.getNames().iterator(); it.hasNext();) { 
      Name n = (Name) it.next();
      n.accept(this,data);
    }
    return data;
  }
  
	public Object visitFixedType(FixedType node, Object data) {
		return data;
	}

	public Object visitArgument(Argument node, Object data) {
	  Type t = node.getType();
    t.accept(this,data);
    return data;
	}

  public Object visitArgumentList(ArgumentList node, Object data ) { 
    for (Iterator it = node.iterator(); it.hasNext(); ) { 
      Argument arg = (Argument) it.next();
      arg.accept(this, data);
    }
    return data;
  }
  
	public Object visitDocComment(DocComment node, Object data) {
		return data;
	}

	public Object visitMethod(Method node, Object data) {
	  node.getArgumentList().accept(this, data);
    node.getThrowsList().accept(this,data);
    if ( node.getFromClause() != null ) { 
      node.getFromClause().accept(this,data);
    }
    return data;
	}


  public Object visitMethodList( MethodList node, Object data ) { 
    for (Iterator i = node.iterator(); i.hasNext(); ) {
      Method m = (Method) i.next();
      m.accept(this, data);
    }
    return data;
  }
  
  
  public Object visitScopedID(ScopedID node, Object data ) { 
    return data;
  }
  
  public Object visitScopedIDList(ScopedIDList node, Object data ) { 
    for (Iterator it = node.iterator(); it.hasNext(); ) { 
      ScopedID id = (ScopedID) it.next();
      id.accept(this, data);
    }
    return data;
  }
  
  public Object visitThrowsList(ThrowsList node, Object data) { 
    for (Iterator it = node.iterator(); it.hasNext(); ) { 
      ScopedID id = (ScopedID) it.next();
      id.accept(this, data);
    }
    return data;
  }
  
  public Object visitFromClause(FromClause node, Object data) { 
    return data;
  }
  
  public Object visitStructType(StructType node, Object data) { 
    for(Iterator i = node.getStructItemList().iterator(); i.hasNext(); ) { 
      StructItem e = (StructItem) i.next();
      e.accept(this, data);
    }
    return data;
  }
  
  public Object visitStructItem(StructItem node, Object data) { 
    return data;
  }
}
